//
//  LPBSConnection.h
//  IMLCoapBase
//
//  Created by 朱永利 on 2018/8/2.
//  Copyright © 2018年 ZhuYongli. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "LPBSBridgeDefine.h"


/**
 Alink错误码
 */
typedef NS_ENUM(NSInteger, LPBSError) {
    LPBSErrorSuccess = 0,                     ///< 业务成功
    LPBSErrorUnknownError = 6201,             ///< 客户端内部错误
    LPBSErrorInvalidParams = 6202,             ///< 入参错误
    LPBSErrorRespError = 6203,             ///< 响应错误
    LPBSErrorRespTimeout = 6204,             ///< 响应超时错误
    LPBSErrorNotAuthorized = 6001,   ///< 需要加密通信，但是认证失败
    LPBSErrorAuthInfoExpired = 6002,   ///< 需要加密通信，认证信息过期，需要刷新
    LPBSErrorConnectTimeout = 6003,
};




typedef NS_ENUM (NSInteger, LPBSConnectionState){
    LPBSConnectionConnected = 1, ///< 设备已经连接上，可以发送业务请求
    LPBSConnectionDisconnected = 2, ///< 设备已断连，不可以发送业务请求
    LPBSConnectionConnecting = 3, ///< 设备连接中
    LPBSConnectionDisconnecting = 4, ///< 设备连接中
};

@protocol LPBSConnectionDelegate <NSObject>

/**
 通告设备连接状态

 @param state 当前连接状态 （LPBSConnectionConnected or LPBSConnectionDisconnected）
 */
- (void)onConnectState:(LPBSConnectionState)state;



/**
 接收到设备上报上来的事件数据

 @param event 事件payload，为设备私有协议格式的数据
 */
- (void)didReceiveEvent:(NSData *)event;
@end



/**
 请求设备服务的结果
 */
@interface LPBSBizResponse : NSObject
@property(nonatomic, assign) BOOL success;  ///<  服务端（智能设备）响应是否成功，成功 dataObject 不为 nil，失败则 responseError 不为 nil
@property (nonatomic, strong, nullable) NSData * dataObject;          ///< 服务端（智能设备）返回的响应数据,一般为 NSData 的数据类型，为设备私有协议格式的数据
@property (nonatomic, strong, nullable) NSError * responseError; ///<  服务端（智能设备）返回的响应错误
@end



typedef void(^ LPBSBizRespCallback)(LPBSBizResponse *);



/**
 跟设备之间的连接，用于向设备发送业务请求，订阅设备事件
 */
@protocol LPBSConnection <NSObject>

/**
 连接状态监听 delegate
 */
@property(nonatomic, weak)id<LPBSConnectionDelegate> connectionDelegate;


/**
 连接状态，请参考 `LPBSConnectionState`
 */
@property(nonatomic, assign)LPBSConnectionState connectionState;

/**
 跟设备进行连接，认证client/server双方身份（如有安全认证逻辑）

 @param authInfo 认证时需要的信息，如果连接时不需要认证，则此处为空
 @param resultBlock 连接是否成功的回调,连接失败的错误码，请参考 `LPBSError`
        比如认证失败 - LPBSErrorAuthInfoExpired。
 */
- (void)startConnect:(NSData *)authInfo result:(LPBSBoolResultBlockExtra)resultBlock;



/**
 断开跟设备的连接
 */
- (void)stopConnect;


/**
 异步请求设备的服务，请求的结果在回调中给出
 请求设备的服务包括两类，一类是对设备物模型中定义的设备属性进行操作，包括属性设置/获取
 另一类是对设备物模型中定义的设备服务进行调用
 @param payload 业务请求的payload,为设备私有协议格式的数据。
 @param options 业务请求的可选参数，包含业务请求的Topic信息等，对于私有协议格式的设备，忽略此参数
 @param resultCallback 业务请求的结果回调，返回的数据格式为设备私有协议格式
 */
- (void)asyncSendRequest:(NSData *)payload
                 options:(NSDictionary *)options
                  result:(LPBSBizRespCallback)resultCallback;


/**
 订阅设备的事件，如属性变更，报警等。
 事件是设备物模型中定义的事件
 当设备事件发生时，需要将事件数据使用 [LPBSConnectionDelegate didReceiveEvent]上报。
 @param resultBlock 订阅的结果回调
 */
- (void)subscribeEvents:(LPBSBoolResultBlock)resultBlock;



/**
 取消订阅设备的事件，如属性变更，报警等。

 @param resultBlock 取消订阅的结果回调
 */
- (void)unsubscribeEvents:(LPBSBoolResultBlock)resultBlock;

@end

